﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Steema.TeeChart.WPF.Styles;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Windows.Markup;
using System.Collections.Specialized;
using System.Collections;
using Steema.TeeChart.WPF;
using System.Diagnostics;
using System.Reflection;
using System.Globalization;
using Steema.TeeChart.WPF.Tools;
using HtCoreInterface;
using System.Data;

namespace XamlControls
{
    [DefaultProperty("Series"), ContentProperty("Series")]
    [Description("Wrapper um Steema.TeeChart.WPF.TChart, der XAML-DependencyProperties zur individuellen Anpassung des Charts bereitstellt.")]
    public partial class Chart : UserControl
    {
        public enum SERIESTYPES
        {
            /// <summary>
            /// Es gilt der Typ der manuell festgelegten Reihen.
            /// </summary>
            NONE,
            /// <summary>
            /// wirksam nur bei global festgelegtem Datenkontext: alle dynamisch angelegten Reihen sind vom Typ "Linie".
            /// </summary>
            LINE,
            /// <summary>
            /// wirksam nur bei global festgelegtem Datenkontext: alle dynamisch angelegten Reihen sind vom Typ "Säule".
            /// </summary>
            BAR,
            /// <summary>
            /// wirksam nur bei global festgelegtem Datenkontext: alle dynamisch angelegten Reihen sind vom Typ "gestapelte Säule".
            /// </summary>
            STACKED_BAR
        }

        private NearestPoint nearestPoint1;
        private Annotation annotation1;

        #region Initialisierung
        static Chart()
        {
        }

        public Chart()
        {
            DataContextChanged += new DependencyPropertyChangedEventHandler(Chart_DataContextChanged);

            InitializeComponent();

            //LicenseManager.CurrentContext.GetSavedLicenseKey(typeof(Steema.TeeChart.WPF.TChart), Assembly.GetExecutingAssembly());
            //LicenseManager.CurrentContext.GetSavedLicenseKey(typeof(Steema.TeeChart.WPF.Chart), Assembly.GetExecutingAssembly());

            tchart.Aspect.View3D = false;
            tchart.Axes.Left.MaximumOffset = 5;
            tchart.Axes.Right.MaximumOffset = 5;
            //tchart.Axes.Left.Labels.ValueFormat = "###,###.00";            
            //tchart.Axes.Right.Labels.ValueFormat = "###,###.00";            
            tchart.Legend.Alignment = LegendAlignments.Top;
            tchart.Legend.LegendStyle = LegendStyles.Series;
            //tchart.Legend.Symbol.OnSymbolDraw += new SymbolDrawEventHandler(Symbol_OnSymbolDraw);
            tchart.Legend.Font.Size = 7;
            tchart.Legend.Visible = false;
            //tchart.BeforeDraw += new PaintChartEventHandler(tchart_BeforeDraw);
            tchart.MouseMove += new MouseEventHandler(tchart_MouseMove);

            nearestPoint1 = new NearestPoint(tchart.Chart);
            nearestPoint1.Active = false;

            annotation1 = new Annotation(tchart.Chart);
            annotation1.Active = false;
            annotation1.TextAlign = TextAlignment.Center;
            annotation1.ClipText = true;
            annotation1.Callout.Visible = true;
            annotation1.Callout.Color = Colors.Blue;
            annotation1.Callout.Arrow.Visible = true;
            annotation1.Callout.Arrow.Color = Colors.Blue;

            Series.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(Series_CollectionChanged);
        }
        #endregion

        #region Reihengenerierung bei Änderung des Datenkontext
        void Chart_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            Steema.TeeChart.WPF.Styles.Bar _b = null;

            if (e.NewValue is DataTable && SeriesType != SERIESTYPES.NONE)
            {
                DataTable _dt = e.NewValue as DataTable;

                if (_dt != null)
                {
                    // bei neuem Datenkontext erstmal alle alten Reihen entfernen, bis auf die deklarativ festgelegten
                    tchart.Series.Clear();

                    for (int _i = 0; _i < _dt.Rows.Count; _i++)
                    {
                        DataRow _dr = _dt.Rows[_i];

                        object _x = _dr[0];

                        // für jede Spalte der Datentabelle eine Reihe anlegen
                        for (int _j = 1; _j < _dt.Columns.Count; _j++)
                        {
                            // Textspalte der Datentabelle selbst nicht mit als Reihe anlegen
                            if (_dt.Columns.IndexOf(TextColumn) != _j)
                            {
                                object _y = _dr[_j];

                                // Reihe noch nicht vorhanden? => dann anlegen
                                if (tchart.Chart.Series.Count < _j)
                                {
                                    Steema.TeeChart.WPF.Styles.Series _s = null;

                                    switch (SeriesType)
                                    {
                                        case SERIESTYPES.LINE:

                                            _s = new Steema.TeeChart.WPF.Styles.Line();

                                            break;

                                        case SERIESTYPES.BAR:

                                            _b = new Steema.TeeChart.WPF.Styles.Bar();

                                            // Rahmen um Säule zeichnen
                                            _b.Pen.Visible = true;
                                            _b.Pen.Width = 0.5;
                                            _b.Pen.Color = Colors.Black;

                                            _s = _b;

                                            break;

                                        case SERIESTYPES.STACKED_BAR:

                                            _b = new Steema.TeeChart.WPF.Styles.Bar();
                                            _b.MultiBar = MultiBars.Stacked;

                                            // Rahmen um Säule zeichnen
                                            _b.Pen.Visible = true;
                                            _b.Pen.Width = 0.5;
                                            _b.Pen.Color = Colors.Black;

                                            _s = _b;

                                            break;
                                    }

                                    // eindeutige Farbe für Reihe setzen, aber nur wenn der Modus "Farbe für jeden Datenpunkt (ColorEach)" nicht gesetzt ist
                                    if (!ColorEach)
                                    {
                                        _s.Color = SeriesColors.GetColorForIndex(_j - 1);
                                    }

                                    // Reihenname entspricht Name der akt. Spalte der Datentabelle (z.B. für Legende)
                                    _s.Title = _dt.Columns[_j].ColumnName;

                                    _s.Marks.Visible = ShowSeriesMarks;

                                    tchart.Chart.Series.Add(_s);
                                }

                                if (_y != null && !(_y is DBNull))
                                {
                                    if (_x is DateTime)
                                    {
                                        if (ColorEach)
                                        {
                                            tchart.Series[_j - 1].Add((DateTime)_x, HtFunctions.AllToDouble(_y), SeriesColors.GetColorForIndex(_i));
                                        }
                                        else
                                        {
                                            tchart.Series[_j - 1].Add((DateTime)_x, HtFunctions.AllToDouble(_y));
                                        }
                                    }
                                    else
                                    {
                                        if (ColorEach)
                                        {
                                            tchart.Series[_j - 1].Add(HtFunctions.AllToDouble(_x), HtFunctions.AllToDouble(_y), SeriesColors.GetColorForIndex(_i));
                                        }
                                        else
                                        {
                                            tchart.Series[_j - 1].Add(HtFunctions.AllToDouble(_x), HtFunctions.AllToDouble(_y));
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                // füllt Reihen ggf. mit Nullwerten auf
                CheckFillUpZeros(this, FillUpZeros);

                // skaliert (Stapel-)Balken auf 100 Prozent
                CheckStacked100(tchart, Stacked100);

                // passt Attribute der Labels an 
                CheckPointLabels(this, TextEach);

                for (int _i = 0; _i < Series.Count; _i++)
                {
                    tchart.Series.Add(Series[_i].SeriesObject);
                }
            }
        }
        #endregion

        #region Legendensymbole zeichnen
        void Symbol_OnSymbolDraw(object sender, SymbolDrawEventArgs e)
        {
            //PathFigure _p = new PathFigure();
            //_p.Segments.Add(new LineSegment(new Point(e.Rect.X, e.Rect.Y), false));
            //_p.Segments.Add(new LineSegment(new Point(e.Rect.X + e.Rect.Width, e.Rect.Y), false));
            //_p.Segments.Add(new LineSegment(new Point(e.Rect.X + e.Rect.Width, e.Rect.Y + e.Rect.Height), false));
            //_p.Segments.Add(new LineSegment(new Point(e.Rect.X, e.Rect.Y + e.Rect.Height), false));

            //tchart.Chart.Graphics3D.DrawPath(new System.Windows.Media.Pen(new SolidColorBrush(Colors.Black), 1.0);

            Rect _r = e.Rect;

            // Rechteck um die Hälfte in X-Richtung verkleinern
            //_r.Inflate(-(e.Rect.Width / 4), 0);
            tchart.Chart.Graphics3D.FillRectangle(new SolidColorBrush(e.Series.Color), (int)_r.X, (int)_r.Y, (int)_r.Width, (int)_r.Height);
        }
        #endregion

        #region ToolTips erstellen bei Mausbewegung
        void tchart_MouseMove(object sender, MouseEventArgs e)
        {
            // Annotation initial unsichtbar schalten
            annotation1.Active = false;
            annotation1.Callout.Visible = false;
            annotation1.Callout.Arrow.Visible = false;

            // Feld mit den dem Mauszeiger nächsten Punkt pro Reihe
            int[] _nearest_points;
            int _series_index, _value_index;
            double _dist, _tmp_dist;
            Point _p1, _p2;

            // aktueller Punkt des Mauszeigers
            System.Windows.Point _mousepos = e.GetPosition(this);
            Rect _chartrect = tchart.Chart.ChartRect;

            // Legendgröße + oberen Leerraum
            double _margintop = tchart.Chart.Legend.Height + 20;

            // befindet sich Mauszeiger innerhalb des Charts ?
            // => nur dann Label anzeigen
            if (_mousepos.Y > _margintop)
            {
                _nearest_points = new int[tchart.Series.Count];
                _p1 = new Point(0, 0);
                _p2 = new Point(_chartrect.Width, _chartrect.Height);

                Point p_near = _p1;
                _series_index = 0;
                _value_index = 0;

                // die nähesten Punkte pro Reihe mithilfe des NearestPoint-Tools bestimmen
                for (int i = 0; i < _nearest_points.Length; i++)
                {
                    nearestPoint1.Series = tchart[i];

                    _nearest_points[i] = nearestPoint1.GetNearestPoint(_mousepos);
                }

                nearestPoint1.Series = null;

                _dist = Distance(_p1, _p2);

                // prüfen, welcher Punkt jeder Reihe am nähesten am Mauspunkt dran liegt
                for (int i = 0; i < _nearest_points.Length; i++)
                {
                    int _index = _nearest_points[i];

                    // Bestimmung der Position des Punktes
                    if (_index >= 0)
                    {
                        _p1.X = tchart[i].CalcXPos(_index);
                        _p1.Y = tchart[i].CalcYPos(_index);
                    }

                    _p2.X = _mousepos.X;
                    _p2.Y = _mousepos.Y;

                    // Entfernung zwischen gefundenem Punkt und Mauszeiger ausrechnen
                    _tmp_dist = Distance(_p1, _p2);

                    // ist gefundener Punkt näher am Mauszeiger als vorheriger ?
                    if ((i == 0) || (_tmp_dist < _dist))
                    {
                        _dist = _tmp_dist;
                        _series_index = i;
                        _value_index = _nearest_points[i];

                        p_near = _p1;
                    }
                }

                // Markierung am nähesten Punkt anzeigen
                annotation1.Callout.XPosition = (int)p_near.X;
                annotation1.Callout.YPosition = (int)p_near.Y;

                if (tchart.Series.Count > _series_index && tchart[_series_index].Count > _value_index)
                {
                    Steema.TeeChart.WPF.Styles.Series _series = tchart[_series_index];

                    // nur anzeigen, wenn Reihe auch aktiv ist, also angezeigt wird
                    if (_series.Active)
                    {
                        SeriesXYPoint _p = _series[_value_index];

                        // Marker und Markerpfeil die Farbe der Reihe geben
                        annotation1.Callout.Color = _series.Color;
                        annotation1.Callout.Arrow.Color = _series.Color;

                        if (_p != null)
                        {
                            try
                            {
                                if (tchart.Axes.Bottom.IsDateTime)
                                {
                                    annotation1.Text = _series.Title + Environment.NewLine + DateTime.FromOADate(_p.X).ToString("dd.MM.yyyy HH:mm:ss.fff") + " " + String.Format("{0:0.###}", _p.Y);
                                }
                                else
                                {
                                    annotation1.Text = _series.Title + Environment.NewLine + String.Format("{0:0.###}", _p.X) + " " + String.Format("{0:0.###}", _p.Y);
                                }

                                if (_mousepos.X > ((_chartrect.Width / 2) + 100))
                                {
                                    annotation1.Left = _mousepos.X - 200;
                                }
                                else
                                {
                                    annotation1.Left = _mousepos.X + 5;
                                }

                                annotation1.Top = _mousepos.Y - 25;

                                // Annotation sichtbar schalten
                                annotation1.Active = true;
                                annotation1.Callout.Visible = true;
                                annotation1.Callout.Arrow.Visible = true;
                            }
                            catch (IndexOutOfRangeException _e)
                            {
                                Trace.WriteLine(_e.Message + Environment.NewLine + _e.StackTrace);
                            }
                        }
                    }
                }
            }
        }

        private double Distance(Point Pt1, Point Pt2)
        {
            double dx, dy;

            dx = Pt1.X - Pt2.X;
            dy = Pt1.Y - Pt2.Y;

            return Math.Sqrt((dx * dx) + (dy * dy));
        }
        #endregion

        #region deklaratives Hinzufügen von Reihen in XAML
        void Series_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            try
            {
                if (e.Action == NotifyCollectionChangedAction.Add)
                {
                    IList _items = e.NewItems;

                    if (_items != null)
                    {
                        lock (_items)
                        {
                            for (int _i = 0; _i < _items.Count; _i++)
                            {
                                object _item = _items[_i];

                                if (_item != null)
                                {
                                    if (_item is Series)
                                    {
                                        Series _newseries = _item as Series;

                                        if (_newseries != null && _newseries.SeriesObject != null)
                                        {
                                            //_newseries.Resources = Resources;
                                            _newseries.DataContext = DataContext;

                                            lock (tchart.Series)
                                            {
                                                if (tchart.Series.IndexOf(_newseries.SeriesObject) < 0)
                                                {
                                                    tchart.Series.Add(_newseries.SeriesObject);
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception _e)
            {
                Trace.WriteLine(_e.Message + " " + _e.StackTrace);
            }
        }
        #endregion

        #region SeriesType
        [Category("Funktion")]
        public SERIESTYPES SeriesType
        {
            get { return (SERIESTYPES)this.GetValue(SeriesTypeProperty); }
            set { this.SetValue(SeriesTypeProperty, value); }
        }

        public static readonly DependencyProperty SeriesTypeProperty = DependencyProperty.Register
        (
          "SeriesType",
          typeof(SERIESTYPES),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              SERIESTYPES.NONE,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onSeriesTypeChanged)
              )
        );

        static void onSeriesTypeChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                DependencyPropertyChangedEventArgs _e = new DependencyPropertyChangedEventArgs(DataContextProperty, _sender_ctrl.DataContext, _sender_ctrl.DataContext);

                _sender_ctrl.Chart_DataContextChanged(sender, _e);
            }
        }
        #endregion

        #region SeriesTypeInt
        [Category("Funktion")]
        [Bindable(true)]
        public int SeriesTypeInt
        {
            get { return (int)this.GetValue(SeriesTypeIntProperty); }
            set { this.SetValue(SeriesTypeIntProperty, value); }
        }

        public static readonly DependencyProperty SeriesTypeIntProperty = DependencyProperty.Register
        (
          "SeriesTypeInt",
          typeof(int),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onSeriesTypeIntChanged)
              )
        );

        static void onSeriesTypeIntChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                _sender_ctrl.SeriesType = (SERIESTYPES)e.NewValue;
            }
        }
        #endregion

        #region Series
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [Category("Funktion")]
        public ObservableCollection<Series> Series
        {
            get
            {
                return (ObservableCollection<Series>)this.GetValue(SeriesProperty);
            }
            set
            {
                this.SetValue(SeriesProperty, value);
            }
        }

        private static readonly DependencyProperty SeriesProperty = DependencyProperty.Register
        (
          "Series",
          typeof(ObservableCollection<Series>),
          typeof(Chart),
          new FrameworkPropertyMetadata
          (
              new ObservableCollection<Series>(),
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault |
              FrameworkPropertyMetadataOptions.AffectsRender |
              FrameworkPropertyMetadataOptions.AffectsArrange,
              new PropertyChangedCallback(OnSeriesChanged)
          )
        );

        static void OnSeriesChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
        }
        #endregion

        #region Clear
        [Category("Funktion")]
        public bool Clear
        {
            get { return (bool)this.GetValue(ClearProperty); }
            set { this.SetValue(ClearProperty, value); }
        }

        public static readonly DependencyProperty ClearProperty = DependencyProperty.Register
        (
          "Clear",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onClearChanged)
              )
        );

        static void onClearChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                if (e.NewValue is bool)
                {
                    if ((bool)e.NewValue)
                    {
                        _sender_ctrl.tchart.Series.Clear();
                    }
                }
            }
        }
        #endregion

        #region DateTimeIncrement
        [Category("Funktion")]
        [Description("ermöglicht hartes Setzen des Zeitachsenintervalls")]
        public DateTimeSteps DateTimeIncrement
        {
            get { return (DateTimeSteps)this.GetValue(DateTimeIncrementProperty); }
            set { this.SetValue(DateTimeIncrementProperty, value); }
        }

        public static readonly DependencyProperty DateTimeIncrementProperty = DependencyProperty.Register
        (
          "DateTimeIncrement",
          typeof(DateTimeSteps),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              DateTimeSteps.None,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onDateTimeIncrementChanged)
              )
        );

        static void onDateTimeIncrementChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep((DateTimeSteps)e.NewValue);
            }
        }
        #endregion

        #region XIncrement
        [Category("Funktion")]
        [Description("ermöglicht hartes Setzen des X-Achsen-Intervalls")]
        public double XIncrement
        {
            get { return (double)this.GetValue(XIncrementProperty); }
            set { this.SetValue(XIncrementProperty, value); }
        }

        public static readonly DependencyProperty XIncrementProperty = DependencyProperty.Register
        (
          "XIncrement",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onXIncrementChanged)
              )
        );

        static void onXIncrementChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                _sender_ctrl.tchart.Axes.Bottom.Increment = HtFunctions.AllToDouble(e.NewValue);
            }
        }
        #endregion

        #region TimeSpanMinutes
        [Category("Funktion")]
        public int TimeSpanMinutes
        {
            get { return (int)this.GetValue(TimeSpanMinutesProperty); }
            set { this.SetValue(TimeSpanMinutesProperty, value); }
        }

        public static readonly DependencyProperty TimeSpanMinutesProperty = DependencyProperty.Register
        (
          "TimeSpanMinutes",
          typeof(int),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              5,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onTimeSpanMinutesChanged)
              )
        );

        static void onTimeSpanMinutesChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is int)
                    {
                        double _val = (int)_data;

                        if (_val >= 0)
                        {
                            if (_val < 0.5)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneSecond);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm:ss.fff";
                            }
                            else if (_val == 0.5)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.FiveSeconds);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm:ss";
                            }
                            else if (_val == 1)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.FifteenSeconds);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm:ss";
                            }
                            else if (_val == 5)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneMinute);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm:ss";
                            }
                            else if (_val == 15)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneMinute);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";
                            }
                            else if (_val == 30)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.FiveMinutes);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";
                            }
                            else if (_val == 60)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.FifteenMinutes);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";
                            }
                            else if (_val == 120)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.ThirtyMinutes);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";
                            }
                            else if (_val == 180)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.ThirtyMinutes);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";
                            }
                            else if (_val == 360)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneHour);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";
                            }
                            else if (_val == 720)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneHour);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";
                            }
                            else if (_val == 1440)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneHour);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";
                            }
                            else if (_val == 2880)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.TwelveHours);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "dd.MM.yyyy HH:mm";
                            }
                            else if (_val == 10080)
                            {
                                _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneDay);
                                _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "dd.MM.yyyy HH:mm";
                            }

                            //DateTime _now = DateTime.Now;
                            //DateTime _past = _now.Subtract(TimeSpan.FromMinutes(_val));

                            //try
                            //{
                            //    _sender_ctrl.tchart.Axes.Bottom.SetMinMax(_past, _now);
                            //}
                            //catch (Exception _e)
                            //{
                            //    Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                            //}
                        }
                    }
                }
            }
        }
        #endregion

        #region View3D
        [Category("Funktion")]
        public bool View3D
        {
            get { return (bool)this.GetValue(View3DProperty); }
            set { this.SetValue(View3DProperty, value); }
        }

        public static readonly DependencyProperty View3DProperty = DependencyProperty.Register
        (
          "View3D",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onView3DChanged)
              )
        );

        static void onView3DChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Aspect.View3D = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region ShowHeader
        [Category("Funktion")]
        public bool ShowHeader
        {
            get { return (bool)this.GetValue(ShowHeaderProperty); }
            set { this.SetValue(ShowHeaderProperty, value); }
        }

        public static readonly DependencyProperty ShowHeaderProperty = DependencyProperty.Register
        (
          "ShowHeader",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onShowHeaderChanged)
              )
        );

        static void onShowHeaderChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Header.Visible = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region ShowLegend
        [Category("Funktion")]
        public bool ShowLegend
        {
            get { return (bool)this.GetValue(ShowLegendProperty); }
            set { this.SetValue(ShowLegendProperty, value); }
        }

        public static readonly DependencyProperty ShowLegendProperty = DependencyProperty.Register
        (
          "ShowLegend",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onShowLegendChanged)
              )
        );

        static void onShowLegendChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Legend.Visible = _val;

                            if (_sender_ctrl.tchart.Series != null)
                            {
                                for (int _i = 0; _i < _sender_ctrl.tchart.Series.Count; _i++)
                                {
                                    _sender_ctrl.tchart.Series[_i].ShowInLegend = _val;
                                }
                            }
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region Title
        [Category("Funktion")]
        public string Title
        {
            get { return (string)this.GetValue(TitleProperty); }
            set { this.SetValue(TitleProperty, value); }
        }

        public static readonly DependencyProperty TitleProperty = DependencyProperty.Register
        (
          "Title",
          typeof(string),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              null,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onTitleChanged)
              )
        );

        static void onTitleChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    string _txt = _data.ToString();

                    try
                    {
                        if (!String.IsNullOrEmpty(_txt))
                        {
                            _sender_ctrl.tchart.Header.Visible = true;
                            _sender_ctrl.tchart.Header.Text = _txt;
                        }
                        else
                        {
                            _sender_ctrl.tchart.Header.Visible = false;
                        }
                    }
                    catch (Exception _e)
                    {
                        Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                    }
                }
            }
        }
        #endregion

        #region XAuto
        [Category("Funktion")]
        public bool XAuto
        {
            get { return (bool)this.GetValue(XAutoProperty); }
            set { this.SetValue(XAutoProperty, value); }
        }

        public static readonly DependencyProperty XAutoProperty = DependencyProperty.Register
        (
          "XAuto",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              true,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onXAutoChanged)
              )
        );

        static void onXAutoChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                CheckMinMax(_sender_ctrl, HtFunctions.AllToBool(e.NewValue));
            }
        }
        #endregion

        #region YLeftAuto
        [Category("Funktion")]
        public bool YLeftAuto
        {
            get { return (bool)this.GetValue(YLeftAutoProperty); }
            set { this.SetValue(YLeftAutoProperty, value); }
        }

        public static readonly DependencyProperty YLeftAutoProperty = DependencyProperty.Register
        (
          "YLeftAuto",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              true,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYLeftAutoChanged)
              )
        );

        static void onYLeftAutoChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Left.Automatic = _val;

                            // wenn Achsen manuell skaliert werden, dann die eingegebenen Grenzen wieder setzen
                            if (!_val)
                            {
                                _sender_ctrl.tchart.Axes.Left.Minimum = _sender_ctrl.YLeftMin;
                                _sender_ctrl.tchart.Axes.Left.Maximum = _sender_ctrl.YLeftMax;
                            }

                            // Chart neu zeichnen - wird nicht veranlasst, bei Setzen des Automatik-Modus !!!
                            _sender_ctrl.tchart.DoInvalidate();
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region YLeftLogarithmic
        [Category("Funktion")]
        public bool YLeftLogarithmic
        {
            get { return (bool)this.GetValue(YLeftLogarithmicProperty); }
            set { this.SetValue(YLeftLogarithmicProperty, value); }
        }

        public static readonly DependencyProperty YLeftLogarithmicProperty = DependencyProperty.Register
        (
          "YLeftLogarithmic",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              true,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYLeftLogarithmicChanged)
              )
        );

        static void onYLeftLogarithmicChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Left.Logarithmic = _val;

                            _sender_ctrl.tchart.DoInvalidate();
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region YRightAuto
        [Category("Funktion")]
        public bool YRightAuto
        {
            get { return (bool)this.GetValue(YRightAutoProperty); }
            set { this.SetValue(YRightAutoProperty, value); }
        }

        public static readonly DependencyProperty YRightAutoProperty = DependencyProperty.Register
        (
          "YRightAuto",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              true,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYRightAutoChanged)
              )
        );

        static void onYRightAutoChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Right.Automatic = _val;

                            // wenn Achsen manuell skaliert werden, dann die eingegebenen Grenzen wieder setzen
                            if (!_val)
                            {
                                _sender_ctrl.tchart.Axes.Right.Minimum = _sender_ctrl.YRightMin;
                                _sender_ctrl.tchart.Axes.Right.Maximum = _sender_ctrl.YRightMax;
                            }

                            // Chart neu zeichnen - wird nicht veranlasst, bei Setzen des Automatik-Modus !!!
                            _sender_ctrl.tchart.DoInvalidate();
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region YRightLogarithmic
        [Category("Funktion")]
        public bool YRightLogarithmic
        {
            get { return (bool)this.GetValue(YRightLogarithmicProperty); }
            set { this.SetValue(YRightLogarithmicProperty, value); }
        }

        public static readonly DependencyProperty YRightLogarithmicProperty = DependencyProperty.Register
        (
          "YRightLogarithmic",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              true,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYRightLogarithmicChanged)
              )
        );

        static void onYRightLogarithmicChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Right.Logarithmic = _val;

                            _sender_ctrl.tchart.DoInvalidate();
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region YLeftMin
        [Category("Funktion")]
        public double YLeftMin
        {
            get { return (double)this.GetValue(YLeftMinProperty); }
            set { this.SetValue(YLeftMinProperty, value); }
        }

        public static readonly DependencyProperty YLeftMinProperty = DependencyProperty.Register
        (
          "YLeftMin",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYLeftMinChanged)
              )
        );

        static void onYLeftMinChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    double _val;

                    if (double.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Left.Minimum = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region YLeftMax
        [Category("Funktion")]
        public double YLeftMax
        {
            get { return (double)this.GetValue(YLeftMaxProperty); }
            set { this.SetValue(YLeftMaxProperty, value); }
        }

        public static readonly DependencyProperty YLeftMaxProperty = DependencyProperty.Register
        (
          "YLeftMax",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYLeftMaxChanged)
              )
        );

        static void onYLeftMaxChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    double _val;

                    if (double.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Left.Maximum = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region YRightMin
        [Category("Funktion")]
        public double YRightMin
        {
            get { return (double)this.GetValue(YRightMinProperty); }
            set { this.SetValue(YRightMinProperty, value); }
        }

        public static readonly DependencyProperty YRightMinProperty = DependencyProperty.Register
        (
          "YRightMin",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYRightMinChanged)
              )
        );

        static void onYRightMinChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    double _val;

                    if (double.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Right.Minimum = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region YRightMax
        [Category("Funktion")]
        public double YRightMax
        {
            get { return (double)this.GetValue(YRightMaxProperty); }
            set { this.SetValue(YRightMaxProperty, value); }
        }

        public static readonly DependencyProperty YRightMaxProperty = DependencyProperty.Register
        (
          "YRightMax",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYRightMaxChanged)
              )
        );

        static void onYRightMaxChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    double _val;

                    if (double.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Right.Maximum = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region XMin
        [Category("Funktion")]
        public double XMin
        {
            get { return (double)this.GetValue(XMinProperty); }
            set { this.SetValue(XMinProperty, value); }
        }

        public static readonly DependencyProperty XMinProperty = DependencyProperty.Register
         (
           "XMin",
           typeof(double),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               double.MinValue,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onXMinChanged)
               )
         );

        static void onXMinChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                CheckMinMax(_sender_ctrl, _sender_ctrl.XAuto);
            }
        }
        #endregion

        #region XMinDate
        [Category("Funktion")]
        public DateTime? XMinDate
        {
            get { return (DateTime?)this.GetValue(XMinDateProperty); }
            set { this.SetValue(XMinDateProperty, value); }
        }

        public static readonly DependencyProperty XMinDateProperty = DependencyProperty.Register
         (
           "XMinDate",
           typeof(DateTime?),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               null,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onXMinDateChanged)
               )
         );

        static void onXMinDateChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is DateTime)
                    {
                        try
                        {
                            double _val = ((DateTime)_data).ToOADate();

                            _sender_ctrl.tchart.Axes.Bottom.Minimum = _val;

                            _sender_ctrl.XMin = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }
        #endregion

        #region XMax
        [Category("Funktion")]
        public double XMax
        {
            get { return (double)this.GetValue(XMaxProperty); }
            set { this.SetValue(XMaxProperty, value); }
        }

        public static readonly DependencyProperty XMaxProperty = DependencyProperty.Register
         (
           "XMax",
           typeof(double),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               double.MaxValue,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onXMaxChanged)
               )
         );

        static void onXMaxChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                CheckMinMax(_sender_ctrl, _sender_ctrl.XAuto);
            }
        }
        #endregion

        #region XMaxDate
        [Category("Funktion")]
        public DateTime? XMaxDate
        {
            get { return (DateTime?)this.GetValue(XMaxDateProperty); }
            set { this.SetValue(XMaxDateProperty, value); }
        }

        public static readonly DependencyProperty XMaxDateProperty = DependencyProperty.Register
         (
           "XMaxDate",
           typeof(DateTime?),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               null,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onXMaxDateChanged)
               )
         );

        static void onXMaxDateChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is DateTime)
                    {
                        try
                        {
                            double _val = ((DateTime)_data).ToOADate();

                            _sender_ctrl.tchart.Axes.Bottom.Maximum = _val;

                            _sender_ctrl.XMax = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        private static void CheckMinMax(Chart aChart, bool isAuto)
        {
            try
            {
                aChart.tchart.Axes.Bottom.Automatic = isAuto;

                if (!isAuto)
                {
                    if (aChart.XMin != double.MinValue && aChart.XMax != double.MaxValue)
                    {
                        aChart.tchart.Axes.Bottom.SetMinMax(aChart.XMin, aChart.XMax);
                    }
                }

                aChart.tchart.DoInvalidate();
            }
            catch (Exception _e)
            {
                Trace.WriteLine(_e.Message + " " + _e.StackTrace);
            }
        }
        #endregion

        #region ShowMarks
        [Category("Funktion")]
        public bool ShowMarks
        {
            get { return (bool)this.GetValue(ShowMarksProperty); }
            set
            { this.SetValue(ShowMarksProperty, value); }
        }

        public static readonly DependencyProperty ShowMarksProperty = DependencyProperty.Register
         (
           "ShowMarks",
           typeof(bool),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               true,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onShowMarksChanged)
               )
         );

        static void onShowMarksChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is bool)
                    {
                        bool _show = (bool)_data;

                        if (_show)
                        {
                            _sender_ctrl.tchart.MouseMove += new MouseEventHandler(_sender_ctrl.tchart_MouseMove);
                        }
                        else
                        {
                            _sender_ctrl.tchart.MouseMove -= new MouseEventHandler(_sender_ctrl.tchart_MouseMove);
                        }

                        _sender_ctrl.annotation1.Active = _show;
                        _sender_ctrl.nearestPoint1.Active = _show;
                    }
                }
            }
        }
        #endregion

        #region ShowSeriesMarks
        [Category("Funktion")]
        public bool ShowSeriesMarks
        {
            get { return (bool)this.GetValue(ShowSeriesMarksProperty); }
            set
            { this.SetValue(ShowSeriesMarksProperty, value); }
        }

        public static readonly DependencyProperty ShowSeriesMarksProperty = DependencyProperty.Register
         (
           "ShowSeriesMarks",
           typeof(bool),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               true,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onShowSeriesMarksChanged)
               )
         );

        static void onShowSeriesMarksChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                bool _show = HtFunctions.AllToBool(e.NewValue);

                foreach (Steema.TeeChart.WPF.Styles.Series _s in _sender_ctrl.tchart.Chart.Series)
                {
                    _s.Marks.Visible = _show;
                }
            }
        }
        #endregion

        #region Stacked100
        [Category("Funktion")]
        public bool Stacked100
        {
            get { return (bool)this.GetValue(Stacked100Property); }
            set
            { this.SetValue(Stacked100Property, value); }
        }

        public static readonly DependencyProperty Stacked100Property = DependencyProperty.Register
         (
           "Stacked100",
           typeof(bool),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               true,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onStacked100Changed)
               )
         );

        static void onStacked100Changed(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                CheckStacked100(_sender_ctrl.tchart, HtFunctions.AllToBool(e.NewValue));
            }
        }

        private static void CheckStacked100(TChart aChart, bool isMode)
        {
            foreach (Steema.TeeChart.WPF.Styles.Series _s in aChart.Chart.Series)
            {
                if (_s is Steema.TeeChart.WPF.Styles.Bar)
                {
                    Steema.TeeChart.WPF.Styles.Bar _b = _s as Steema.TeeChart.WPF.Styles.Bar;

                    if (_b.MultiBar == MultiBars.Stacked || _b.MultiBar == MultiBars.Stacked100)
                    {
                        _b.MultiBar = (isMode) ? MultiBars.Stacked100 : MultiBars.Stacked;
                    }
                }
            }
        }

        #endregion

        #region ShowRightAxisGridLines
        [Category("Funktion")]
        public bool ShowRightAxisGridLines
        {
            get { return (bool)this.GetValue(ShowRightAxisGridLinesProperty); }
            set { this.SetValue(ShowRightAxisGridLinesProperty, value); }
        }

        public static readonly DependencyProperty ShowRightAxisGridLinesProperty = DependencyProperty.Register
         (
           "ShowRightAxisGridLines",
           typeof(bool),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               true,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onShowRightAxisGridLinesChanged)
               )
         );

        static void onShowRightAxisGridLinesChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is bool)
                    {
                        _sender_ctrl.tchart.Axes.Right.Grid.Visible = (bool)_data;
                    }
                }
            }
        }
        #endregion

        #region ShowBottomAxisGridLines
        [Category("Funktion")]
        public bool ShowBottomAxisGridLines
        {
            get { return (bool)this.GetValue(ShowBottomAxisGridLinesProperty); }
            set { this.SetValue(ShowBottomAxisGridLinesProperty, value); }
        }

        public static readonly DependencyProperty ShowBottomAxisGridLinesProperty = DependencyProperty.Register
         (
           "ShowBottomAxisGridLines",
           typeof(bool),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               true,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onShowBottomAxisGridLinesChanged)
               )
         );

        static void onShowBottomAxisGridLinesChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is bool)
                    {
                        _sender_ctrl.tchart.Axes.Bottom.Grid.Visible = (bool)_data;
                    }
                }
            }
        }
        #endregion

        #region FillUpZeros
        [Category("Funktion")]
        public bool FillUpZeros
        {
            get { return (bool)this.GetValue(FillUpZerosProperty); }
            set
            { this.SetValue(FillUpZerosProperty, value); }
        }

        public static readonly DependencyProperty FillUpZerosProperty = DependencyProperty.Register
         (
           "FillUpZeros",
           typeof(bool),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               false,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onFillUpZerosChanged)
               )
         );


        static void onFillUpZerosChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _c = sender as Chart;

            if (_c != null)
            {
                CheckFillUpZeros(_c, HtFunctions.AllToBool(e.NewValue));
            }
        }

        private static void CheckFillUpZeros(Chart aChart, bool isMode)
        {
            if (isMode)
            {
                Steema.TeeChart.WPF.Chart _intChart = aChart.tchart.Chart;

                ChartUtil.FillUpZeros(_intChart, _intChart.Axes.Bottom.Increment, aChart.XMin, aChart.XMax);
            }
        }
        #endregion

        #region ShowLeftAxisTitle
        [Category("Funktion")]
        public bool ShowLeftAxisTitle
        {
            get { return (bool)this.GetValue(ShowLeftAxisTitleProperty); }
            set
            { this.SetValue(ShowLeftAxisTitleProperty, value); }
        }

        public static readonly DependencyProperty ShowLeftAxisTitleProperty = DependencyProperty.Register
         (
           "ShowLeftAxisTitle",
           typeof(bool),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               false,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onShowLeftAxisTitleChanged)
               )
         );


        static void onShowLeftAxisTitleChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _c = sender as Chart;

            if (_c != null)
            {
                _c.tchart.Axes.Left.Title.Visible = HtFunctions.AllToBool(e.NewValue);
            }
        }
        #endregion

        #region LeftAxisTitle
        [Category("Funktion")]
        public string LeftAxisTitle
        {
            get { return (string)this.GetValue(LeftAxisTitleProperty); }
            set
            { this.SetValue(LeftAxisTitleProperty, value); }
        }

        public static readonly DependencyProperty LeftAxisTitleProperty = DependencyProperty.Register
         (
           "LeftAxisTitle",
           typeof(string),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               String.Empty,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onLeftAxisTitleChanged)
               )
         );

        static void onLeftAxisTitleChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _c = sender as Chart;

            if (_c != null)
            {
                _c.tchart.Axes.Left.Title.Text = HtFunctions.AllToString(e.NewValue);
            }
        }
        #endregion

        #region ColorEach
        [Category("Funktion")]
        public bool ColorEach
        {
            get { return (bool)this.GetValue(ColorEachProperty); }
            set { this.SetValue(ColorEachProperty, value); }
        }

        public static readonly DependencyProperty ColorEachProperty = DependencyProperty.Register
        (
          "ColorEach",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onColorEachChanged)
              )
        );

        static void onColorEachChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                for (int _i = 0; _i < _sender_ctrl.tchart.Series.Count; _i++)
                {
                    Steema.TeeChart.WPF.Styles.Series _s = _sender_ctrl.tchart.Series[_i];

                    bool _ismode = HtFunctions.AllToBool(e.NewValue);

                    _s.ColorEach = _ismode;

                    if (_ismode)
                    {
                        for (int _j = 0; _j < _s.Count; _j++)
                        {
                            _s[_j].Color = SeriesColors.GetColorForIndex(_j);
                        }
                    }
                }
            }
        }
        #endregion

        #region TextEach
        [Category("Funktion")]
        [Description("Soll für jeden Datenpunkt eine Beschriftung verwendet werden? Ja, dann unter TextColumn den Namen der Spalte der Datentabelle angeben, aus der der Text gelesen werden soll.")]
        public bool TextEach
        {
            get { return (bool)this.GetValue(TextEachProperty); }
            set { this.SetValue(TextEachProperty, value); }
        }

        public static readonly DependencyProperty TextEachProperty = DependencyProperty.Register
        (
          "TextEach",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onTextEachChanged)
              )
        );

        static void onTextEachChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            CheckPointLabels(sender as Chart, HtFunctions.AllToBool(e.NewValue));
        }
        #endregion

        #region TextColumn
        [Category("Funktion")]
        [Description("Name der Spalte der Datentabelle, aus der die Bezeichnung für jeden Datenpunkt gelesen wird")]
        public string TextColumn
        {
            get { return (string)this.GetValue(TextColumnProperty); }
            set { this.SetValue(TextColumnProperty, value); }
        }

        public static readonly DependencyProperty TextColumnProperty = DependencyProperty.Register
        (
          "TextColumn",
          typeof(string),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              string.Empty,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onTextColumnChanged)
              )
        );

        static void onTextColumnChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _c = sender as Chart;

            CheckPointLabels(_c, _c.TextEach);
        }

        private double oldFontSize = double.MinValue;

        private static void CheckPointLabels(Chart aChart, bool textEach)
        {
            if (aChart != null && aChart.DataContext != null && aChart.DataContext is DataTable)
            {
                aChart.tchart.GetAxisLabel -= new GetAxisLabelEventHandler(tchart_GetAxisLabel);

                AxisLabels _lbls = aChart.tchart.Axes.Bottom.Labels;
                _lbls.MultiLine = false;
                _lbls.Angle = 0;

                if (aChart.oldFontSize != double.MinValue)
                {
                    _lbls.Font.Size = aChart.oldFontSize;
                }

                if (textEach)
                {
                    if (!aChart.ColorEach)
                    {
                        // alte Schriftgröße zwischenspeichern
                        aChart.oldFontSize = _lbls.Font.Size;

                        aChart.tchart.GetAxisLabel += new GetAxisLabelEventHandler(tchart_GetAxisLabel);

                        _lbls.Angle = 45;
                        _lbls.Font.Size = 6;
                        _lbls.MultiLine = true;
                    }

                    // nur Text anzeigen (statt Wert Datenpunkt + Text)
                    aChart.tchart.Legend.TextStyle = LegendTextStyles.Plain;

                    DataTable _dt = aChart.DataContext as DataTable;

                    if (_dt.Columns.IndexOf(aChart.TextColumn) > -1)
                    {
                        for (int _i = 0; _i < aChart.tchart.Series.Count; _i++)
                        {
                            Steema.TeeChart.WPF.Styles.Series _s = aChart.tchart.Series[_i];

                            for (int _j = 0; _j < _s.Count; _j++)
                            {
                                if (_dt.Rows.Count > _j)
                                {
                                    _s[_j].Label = HtFunctions.AllToString(_dt.Rows[_j][aChart.TextColumn]);
                                }
                            }
                        }
                    }
                }

                //aChart.tchart.Legend.TextStyle = (LegendTextStyles)aChart.LegendTextStyleInt;
            }
        }

        static void tchart_GetAxisLabel(object sender, GetAxisLabelEventArgs e)
        {
            Steema.TeeChart.WPF.Axis _axis = sender as Steema.TeeChart.WPF.Axis;

            if (_axis != null && _axis == _axis.Chart.Axes.Bottom)
            {
                e.LabelText = ChartUtil.WrapText(e.LabelText);
            }
        }

        #endregion

        #region XAxisLabelAuto
        [Category("Funktion")]
        public bool XAxisLabelAuto
        {
            get { return (bool)this.GetValue(XAxisLabelAutoProperty); }
            set { this.SetValue(XAxisLabelAutoProperty, value); }
        }

        public static readonly DependencyProperty XAxisLabelAutoProperty = DependencyProperty.Register
        (
          "XAxisLabelAuto",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              true,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onXAxisLabelAutoChanged)
              )
        );

        static void onXAxisLabelAutoChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _c = sender as Chart;

            if (_c != null)
            {
                if (HtFunctions.AllToBool(e.NewValue))
                {
                    _c.tchart.Axes.Bottom.Labels.Style = AxisLabelStyle.Auto;
                }
                else
                {
                    _c.tchart.Axes.Bottom.Labels.Style = AxisLabelStyle.Value;
                }
            }
        }
        #endregion

        #region LegendStyleInt
        [Category("Funktion")]
        [Description(@"Auto = 0,
        Series = 1,
        Values = 2,
        LastValues = 3,
        Palette = 4")]
        [Bindable(true)]
        public int LegendStyleInt
        {
            get { return (int)this.GetValue(LegendStyleIntProperty); }
            set { this.SetValue(LegendStyleIntProperty, value); }
        }

        public static readonly DependencyProperty LegendStyleIntProperty = DependencyProperty.Register
        (
          "LegendStyleInt",
          typeof(int),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onLegendStyleIntChanged)
              )
        );

        static void onLegendStyleIntChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                _sender_ctrl.tchart.Legend.LegendStyle = (LegendStyles)e.NewValue;
            }
        }
        #endregion

        #region LegendTextStyleInt
        [Category("Funktion")]
        [Description(@"Plain = 0,
        LeftValue = 1,
        RightValue = 2,
        LeftPercent = 3,
        RightPercent = 4,
        XValue = 5,
        Value = 6,
        Percent = 7,
        XAndValue = 8,
        XAndPercent = 9")]
        [Bindable(true)]
        public int LegendTextStyleInt
        {
            get { return (int)this.GetValue(LegendTextStyleIntProperty); }
            set { this.SetValue(LegendTextStyleIntProperty, value); }
        }

        public static readonly DependencyProperty LegendTextStyleIntProperty = DependencyProperty.Register
        (
          "LegendTextStyleInt",
          typeof(int),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              (int)LegendTextStyles.LeftValue,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onLegendTextStyleIntChanged)
              )
        );

        static void onLegendTextStyleIntChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                _sender_ctrl.tchart.Legend.TextStyle = (LegendTextStyles)e.NewValue;
            }
        }
        #endregion

    }
}
