﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;

namespace XamlControls
{
    public sealed class ChartUtil
    {
        /// <summary>
        /// Füllt alle Reihen des Charts mit Nullpunkten auf, an allen Stellen wo kein Wert vorliegt.
        /// Damit werden z.B. Säulen an X-Stellen immer mit gleicher Breite nebeneinander dargestellt.
        /// </summary>
        /// <param name="aChart">das Chart-Objekt</param>
        /// <param name="aIncrement">die Schrittweite, bei der auf vorhandene Werte geprüft wird</param>
        /// <param name="aMin">Minimum</param>
        /// <param name="aMax">Maximum</param>
        public static void FillUpZeros(Steema.TeeChart.WPF.Chart aChart, double aIncrement, double aMin, double aMax)
        {
            aChart.AutoRepaint = false;

            foreach (Steema.TeeChart.WPF.Styles.Series _s in aChart.Series)
            {
                // jeden Zeitwert durchgehen und 0-Wert einfügen, falls für diesen Zeitwert noch kein
                // Wertepaar vorhanden ist
                for (double _i = aMin; _i < aMax; _i += aIncrement)
                {
                    if (_s.XValues.IndexOf(_i) == -1)
                    {
                        _s.Add(_i, 0, Colors.Transparent);
                    }
                }

                //_s.Repaint();
            }

            aChart.AutoRepaint = true;
        }

        public static string WrapText(string aText)
        {
            // nach Bindestrichen suchen
            string[] _ar = aText.Split('-');

            string _newtxt = String.Empty;

            if (_ar != null)
            {
                string _substr = string.Empty;
                bool _appended = false;

                for (int _i = 0; _i < _ar.Length; _i++)
                {
                    _substr = _ar[_i];

                    _appended = false;

                    AutoWrapText(ref _substr, 5);

                    while (_substr.Length < 5 && (_i + 1) < _ar.Length)
                    {
                        string _s = _ar[_i + 1];

                        AutoWrapText(ref _s, 5);

                        _substr += "-" + _s;

                        _i++;

                        _appended = true;
                    }

                    if (_i > 0 && !_appended)
                    {
                        _newtxt += "-\n" + _substr;
                    }
                    else
                    {
                        _newtxt = _substr;
                    }
                }
            }

            return _newtxt;
        }

        private static bool AutoWrapText(ref string aText, int aMaxCharCount)
        {
            int _i = aMaxCharCount;
            string _append_str = "-\n";

            bool _wrapped = false;

            while (_i < aText.Length)
            {
                _wrapped = true;

                aText = aText.Insert(_i, _append_str);

                _i += (aMaxCharCount + _append_str.Length);
            }

            return _wrapped;
        }
    }
}
